#include "genesis.h"
#include "control.h"

#define JOY_REDUNDANCE	( 5 )

/********************
 TO DO
 
 ********************/


struct Joy Joypad1;
struct Joy Joypad2;

/********************************** PRIVATE *****************************/


/********************* INIT_JOYPADS *************
 *  Initialize Joypads                          *
 *                                              *
 *   Made by P.Lee                              *
 ************************************************/
static void init_joypad()
{
    register volatile uchar *pb;

    pb = (uchar *) 0xa10009;
    *pb = 0x40;
    pb += 2;
    *pb = 0x40;
    pb += 2;
    *pb = 0x40;
}

/*********************** READ_JOYPAD1 *******************
 *  Read joypad #1                                      *
 *  Return a byte like this :                           *
 *                              76543210                *
 *                              SACBRLDU                *
 *  A = button A                                        *
 *  B = button B                                        *
 *  C = button C                                        *
 *  D = Down                                            *
 *  L = Left                                            *
 *  R = Right                                           *
 *  U = Up                                              *
 *  S = Start                                           *
 *                                                      *
 *  Made by P.Lee                                       *
 *      to be modified for 6 buttons use                *
*********************************************************/
static ushort read_joypad1()
{
    register volatile uchar *pb;
    ushort i, j;

    pb = (uchar *) 0xa10003;

    *pb = 0x40;        /* check joypad */
    asm("nop");
    asm("nop");
    i = *pb & 0x3f;

    *pb = 0;           /* check buttons */
    asm("nop");
    asm("nop");
    j = (*pb & 0x30) << 2;

    return( ~(i|j) );
}

/************************** READ_JOYPAD2 ****************
 * see read_joypad1 for more details                    *
 ********************************************************/
static ushort read_joypad2()
{
    register volatile uchar *pb;
    ushort i, j;

    pb = (uchar *) 0xa10005;

    *pb = 0x40;        /* check joypad */
    asm("nop");
    asm("nop");
    i = *pb & 0x3f;

    *pb = 0;           /* check buttons */
    asm("nop");
    asm("nop");
    j = (*pb & 0x30) << 2;

    return(~(i | j));
}

/************************** JoyRead *********************
 * read if joy change or still down 										*
 ********************************************************/
static void JoyRead( ushort joy, struct Joy *Joypad)
{
	//ushort joy;
	uchar newBut, newDir, oldBut;
	
	newBut = 0;
	//joy = read_joypad1();			

	if (joy)
	{
		newBut = (uchar) (joy&0xF0);
		if (newBut)
		{
				// specific case : RESET
				if (newBut == (BUTTON_A+BUTTON_B+BUTTON_C+BUTTON_S))
				{
     				ResetState = 1;
     				return;
				}
				newBut &= ~Joypad->keydown;
		}

		Joypad->key = newBut;	
		Joypad->keydown= (uchar) (joy&0xF0);

		// specific case : PAUSE
		if (newBut == BUTTON_S)
		{
   		PauseState = !PauseState;
   		return;
		}

		if ( Joypad->wait )
		{
			Joypad->wait--;
			return;
		}
		
		newDir = (uchar) (joy&0x0F);
		if (newDir)	Joypad->key = (newBut|newDir);		

		Joypad->wait = 0;
		if (joy)	Joypad->wait = Joypad->initval;
	}
}




/******************************** PUBLIC *********************************/
/********************** Ctrl_Init ***********************
 *  Init Joy refresh rate												      	*
 *                                                      *
 *  Made by Kaneda                                      *
 ********************************************************/
void Ctrl_Init( uchar redundance )
{
	ushort joy;
	
	init_joypad();	
	
	PauseState = 0;
	ResetState = 0;
		
	Joypad1.key=0x00;
	Joypad1.keydown=0x00;
	Joypad1.wait=0;
	Joypad1.initval=redundance;

	Joypad2.key=0x00;
	Joypad2.keydown=0x00;
	Joypad2.wait=0;
	Joypad2.initval=redundance;	
}

/********************** Ctrl_Read ***********************
 *  Read the 2 joy data													      	*
 *	access data throught Joypad1&Joypad2
 *                                                      *
 *  Made by Kaneda                                      *
 ********************************************************/
void Ctrl_Read( )
{
	JoyRead( read_joypad1(), &Joypad1);
	JoyRead( read_joypad2(), &Joypad2);
}

/********************** Ctrl_Wait **********************
 *  Wait for a button to be pressed                     *
 *                                                      *
 *  Button can be A, B, C or Start not direction!!      *
 *	Ctrl_Init was called befoire
 *                                                      *
 *  Made by Kaneda                                      *
 ********************************************************/
void Ctrl_Wait( )
{
    ushort i;

		i = 0;
    while(i == 0)
    {
        Ctrl_Read( );
        i = ((Joypad1.key&JOY_BUTTONS) || (Joypad2.key&JOY_BUTTONS) );
    }
}