#include "genesis.h"
#include "control.h"
#include "video.h"
#include "z80.h"
#include "debug.h"
#include "mappy.h"

#include "rsc_s.h"

extern struct mappyResource mvst;
struct mappyLevel 		levelMap;

void (*UpdateFrame) (void);

/*****************  Initialize GFX  ****************/
/* Read the comment for more info on each register */
/***************************************************/
void Init_GFX( )
{
    volatile register ushort *pw;
    pw = (ushort *) GFXCNTL;

    *pw = 0x8004;   /* reg. 0 - disEnable HBL */
    *pw = 0x8174;   /* reg. 1 - Enable display, VBL, DMA + VCell size (28) */
    *pw = 0x8230;   /* reg. 2 - Plane A =$30*$400=$C000 */
    *pw = 0x832C;   /* reg. 3 - Window  =$2C*$400=$B000 */
    *pw = 0x8407;   /* reg. 4 - Plane B =$7*$2000=$E000 */
    *pw = 0x855F;   /* reg. 5 - sprite table begins at $BE00=$5F*$200 */
    *pw = 0x8600;   /* reg. 6 - not used */
    *pw = 0x8700;   /* reg. 7 - Background Color number*/
    *pw = 0x8800;   /* reg. 8 - not used */
    *pw = 0x8900;   /* reg. 9 - not used */
    *pw = 0x8a00;   /* reg 10 - HInterrupt timing */
    *pw = 0x8b00;   /* reg 11 - $0000abcd a=extr.int b=vscr cd=hscr */
    *pw = 0x8c00;   /* reg 12 - hcell mode + shadow/highight + interlaced mode */
    *pw = 0x8d2E;   /* reg 13 - HScoll Table = $B800 */
    *pw = 0x8e00;   /* reg 14 - not used */
    *pw = 0x8f02;   /* reg 15 - auto increment data */
    *pw = 0x9010;   /* reg 16 - scrl screen v&h size */
    Video_ScreenSize( 32, 64 );
   
    *pw = 0x9100;   /* reg 17 - window hpos */
    *pw = 0x9200;   /* reg 18 - window vpos */
};

/**/
void Int_Error( )
{
	Debug_Alert("Interrupt error");	
}

void VBlank_sub( )
{
	vtimer++;
	UpdateFrame( );	
}

/**/

void UpdateFrame_Nothing( )
{
	
}

void UpdateFrame_Game( )
{
	Ctrl_Read( );
}

void DrawPad( ushort x, ushort y, ushort tileIdx)
{
	Video_Tile(x+0, y+0, tileIdx+0, APLAN);
	Video_Tile(x+1, y+0, tileIdx+2, APLAN);
	Video_Tile(x+2, y+0, tileIdx+4, APLAN);
	Video_Tile(x+3, y+0, tileIdx+6, APLAN);
	Video_Tile(x+0, y+1, tileIdx+1, APLAN);
	Video_Tile(x+1, y+1, tileIdx+3, APLAN);
	Video_Tile(x+2, y+1, tileIdx+5, APLAN);
	Video_Tile(x+3, y+1, tileIdx+7, APLAN);
}

void DrawString( uchar x, uchar y, uchar *sText )
{
	ushort tile;
	
	tile = 0xA000 - 32; //priority, pal1
	
	while( sText[0]!= 0 )
	{
		Video_Tile(  x++, y, tile + sText[0], APLAN);
		*sText++;
	}
}



int main( )
{
	UpdateFrame = UpdateFrame_Nothing;
	Init_GFX( );
	Video_Init();
	Z80_LoadDriver( );
	
	Video_DoDMA_copy((uint) &charset, 0, 96*32 );
	Video_DoDMA_copy((uint) &pad_tile, 96*32, 8*32);
	
	DrawPad(4,  6, 96);
	DrawString(9, 6, "Press A for FM music");
	
	DrawPad(4, 10, 96);
	DrawString(9, 10, "Press B for DAC sound");

	DrawPad(4, 14, 96);
	DrawString(9, 14, "Press C for PSG fx");

	DrawPad(4, 18, 96);
	DrawString(9, 18, "Press START to mute");		

	DrawString(  0, 23, "Thanx Fonzie, Pascal, Stef, TmEE");		
	DrawString( 3, 24, "Logo by Pascal, pad by ob1");		
	DrawString( 9, 25, "FM by Vanisher");		

	levelMap.data = &mvst;
	levelMap.idxPal = PAL1;
	levelMap.idxTile = 96+8;
	levelMap.idxLayer = 0;
	levelMap.fieldAdr = APLAN;
	levelMap.initX = 2;
	levelMap.initY = 0;	
	
	mappy_Load(&levelMap);		
	Video_SetPal( PAL0, pad_pal);	

	Ctrl_Init( 0 );
	UpdateFrame = UpdateFrame_Game;

	while(1)
	{
		wait_sync();
		
		if (Joypad1.key == BUTTON_A)
		{
			
			Z80_TempoFM( 5 );
			Z80_PlayFM( &FM_Lib[0] );
		}
		else if (Joypad1.key == BUTTON_B)
		{
			Z80_DisablePSG( 1 );
			Z80_PlayDAC( &DAC_Lib[0], 11882 );
		}
		else if (Joypad1.key == BUTTON_C)
		{
			Z80_EnablePSG( 1 );
			Z80_PlayPSG( &PSG_Lib[0] );
		}
		else if (Joypad1.key == BUTTON_S)
		{
			Z80_StopFM( );
			Z80_StopDAC( );
			Z80_StopPSG( );
		}
}	
	
	return (0);
} 