#include "genesis.h"
#include "z80.h"
#include "video.h"
#include "sprite.h"
#include "debug.h"


// TO DO : check valid data in Sprite_Show
// TO DO : handle big sprites (multisprites)

ushort  KANGEN_sprites_def[40*5]; /* 40 sprites max */

/***************** Sprite_Add *******************
 * Definition of the sprite                     *
 * number = number of the sprite                *
 * posx, posy =position of the sprite           *
 * size = in hexa                               *
 *        ex: 0xb00 = 3*4 (b=1011)              *
 * from =                                       *
 *      bit 15= priority                        *          
 *      bit 14-13 = palette                     *          
 *      bit 12 = vertical flipping              *          
 *      bit 11 = horizontal flipping            *          
 *      bit 10-0 = tile number                  *
 *                                              *
 *      Made by Kaneda                          *
 ************************************************/
BOOL Sprite_Add(uchar number,ushort posx,ushort posy,ushort size,ushort from)
{
        /* As you can see, all is based on a uint array         */
        /* It's my choice, but you can find something else      */
        ushort loc;
        
        if ( number >= 40 )	return FALSE;
        
        loc=5*number;
        KANGEN_sprites_def[loc]=posy&0x03FF;
        KANGEN_sprites_def[loc+1]=size&0x0F00;
        KANGEN_sprites_def[loc+2]=number&0x007F;
        KANGEN_sprites_def[loc+3]=from&0xFFFF;
        KANGEN_sprites_def[loc+4]=posx&0x03FF;
        
        return TRUE;
}

/*********************** Sprite_Show ********************
 *  Show the sprites                                    *
 *  from = number of the first sprite to show           *
 *  to   =   "    "   "  last    "    "   "             *
 *                                                      *
 *    Made by P.Lee                                     *
 *    Modified by Kaneda to use SPRITES_DEF[]           *
 ********************************************************/
void Sprite_Show(uchar from, uchar to)
{
    register uint *pl;
    register ushort *pw;
    register ushort i;

    /* Sprite table begins at (see GENESIS.H)  */
    /* Note: We set this address using         */
    /*       reg. 85 in the init_gfx routine. */
    pl = (uint *) GFXCNTL;
    *pl = GFX_WRITE_ADDR(SLIST);

    pw = (ushort *) GFXDATA;    

    for ( i = from ; i < to ; i++ )
    {
            /* Set the y position */
            *pw = KANGEN_sprites_def[5*i];
        
            /* We need to set the size and number of the */
            /* sprite we want to show.  0=last, 1=first, */
            /* 2=second, etc...                          */
            /* 0x500 = 2x2 tile sprite                   */
            /* 0 = last sprite                           */
            if (i == (to-1))
            	*pw = KANGEN_sprites_def[5*i +1]; 
            else
            	*pw = (KANGEN_sprites_def[5*i +1] | (i+1)); 
        
            /* bit 15= priority                          */
            /* bit 14-13 = palette                       */
            /* bit 12 = vertical flipping                */
            /* bit 11 = horizontal flipping              */
            /* bit 10-0 = tile number                    */
            /* The genesis shows sprites x then y.       */
            /* Note: Look at the tile data.              */
            *pw = KANGEN_sprites_def[5*i +3];
        
            /* x position */
            *pw = KANGEN_sprites_def[5*i +4];
    }
}

/********************* Sprite_Delete ********************
 * Reset of the nbr first sprites                       *
 *                                                      *
 *  Made by Kaneda                                      *
 *	0.1	correct a big error!			*
 ********************************************************/
void Sprite_Delete( uchar nbr)
{
        register uint *pl;
        register volatile ushort *pw;
        ushort i;

		nbr*=5;
		
        pl = (uint *) GFXCNTL;
        *pl = GFX_WRITE_ADDR(SLIST);

        pw = (ushort *) GFXDATA;

        for (i= 0 ; i < nbr ; i++)
                *pw = 0x00;
}

