#ifndef __INC_VIDEO__
#define __INC_VIDEO__

/*
 *      GFX Registers
 */
#define GFXDATA 0xc00000
#define GFXCNTL 0xc00004


/*
 *      GFX address macros
 */
#define GFX_WRITE_ADDR(x) ((0x4000 + (x & 0x3fff)) << 16) + (x >> 14)
#define GFX_DMA_ADDR(x)   ((0x4000 + (x & 0x3fff)) << 16) + ((x >> 14) | 0x80)
#define GFX_COLOR_WRITE_ADDR(x) ((0xc000 + (x & 0x3fff)) << 16) + (x >> 14)
#define GFX_COLOR_READ_ADDR(x)  ((0x0000 + (x & 0x007f)) << 16) + (x >> 14) + 0x0020

#define GFX_VERT_SCROLL(x) ((0x4000 + (x & 0x3fff)) << 16) + ((x >> 14) | 0x10)
#define GFX_HORZ_SCROLL(x) ((0x4000 + (x & 0x3fff)) << 16) + (x >> 14)   

/*
 *  Field Adr define
 */

#define WPLAN 0xF000 /* mask : 0xF000 | Depend of window size...for a 32x32, 2048 max / 0x800 */
#define APLAN 0xC000 /* mask : 0xE000 | in 64*64, 4096b / 0x1000 */
#define BPLAN 0xE000 /* mask : 0xE000 | in 64*64, 4096b / 0x1000 */
#define SLIST 0xBE00 /* mask : 0xFC00 | max size 640b / 0x280 => 80 sprites */
#define HSCRL 0xB800 /* mask : 0xFC00 | B800 - 0000 = B800 /32 = 1472 tiles max | size max 1024b / 0x400 */


#define	PAL0	0
#define	PAL1	1
#define	PAL2	2
#define	PAL3	3

// to use in Video_Fill()
#define	NO_FLIP	0
#define	FLIPV	1
#define FLIPH	2

void Video_Init();

void Video_ScreenSize( ushort width, ushort height);

void Video_DoDMA_copy(uint source, ushort dest, ushort len);

void Video_Tile(ushort posx, ushort posy, ushort tileDef, ushort fieldAdr);
void Video_Fill(ushort from, ushort to, uchar pal, ushort posx, ushort posy, ushort fieldAdr, uchar pri, uchar flipV, uchar flipH);

void Video_SetPal(uchar palNumber,ushort *palData);
void Video_FadeIn(ushort *palDest,uchar palNum);

void Video_VScroll(short int posA,short int posB);
void Video_HScroll(short int posA,short int posB);

#endif
