	.equ	Z80_BUSREQ,0xa11100
	.equ	Z80_RESET,0xa11200
	
	.equ	FM_CMD,0xa01500
	.equ  FM_TEMPO, 0xa01514
	.equ	FM_ADR,0xa0151E
	
	
	.equ	DAC_CMD, 0xa01521
	.equ	DAC_ADR, 0xa01524
	.equ  DAC_SIZE,0xa01527

	.equ	PSG_CMD, 0xa01529
	.equ	PSG_CHAN, 0xa0152A
	.equ	PSG_ADR, 0xa0152B


PAUSE	MACRO
		move.w  #0x100,Z80_BUSREQ
	ENDM

RESET MACRO
		move.w  #0x100,Z80_RESET
	ENDM

RELEASE MACRO
	move.w  #0x0,Z80_BUSREQ
	ENDM

; *******************************************************************************
; FUNCTIONS:	Z80 handler
; DESCRIPTION:	main control of the z80
; *******************************************************************************	
	.globl Z80_Pause
Z80_Pause:		
	PAUSE      

; *******************************************************************************

	.globl Z80_Reset
Z80_Reset:
	RESET		

; *******************************************************************************
	.globl Z80_Release
Z80_Release:
	RELEASE
        
; *******************************************************************************
	.align 1
	.globl Z80_LoadDriver
Z80_LoadDriver:
	PAUSE
	RESET
	
	lea     Z80Driver,%a0
	lea     0xa00000,%a1
	move.l  #Z80DriverEnd,%d0
	move.l  #Z80Driver,%d1
	sub.l  	%d1,%d0
Z80loop:
	move.b  (%a0)+,(%a1)+
	subq.w  #1,%d0
	bne     Z80loop

	RELEASE
	rts

; *******************************************************************************
; FUNCTIONS:	FM handler
; DESCRIPTION:	control of the FM part
; *******************************************************************************	
	.align 1
	.globl Z80_PlayFM
Z80_PlayFM:
	PAUSE
	movea.l  4(%sp), %a0 
	move.l %a0, %d0
	add.l	#0x200, %d0 ;skip 0x200 bytes
	move.l #FM_ADR, %a0
	move.b %d0,(%a0)+
	lsr.l #8,%d0
	move.b %d0,(%a0)+
	lsr.l #8,%d0
	move.b %d0,(%a0)

	move.l #FM_CMD,%a0
	move.b #1,(%a0)+ ; 0 :silence 1: loop play 2: play once
	move.b #0xf,(%a0) ; reset previous command

	RELEASE
	rts	

; *******************************************************************************

	.align 1
	.globl Z80_StopFM
Z80_StopFM:
	PAUSE
	move.l #FM_CMD,%a0
	move.b #0,(%a0)+ ; 0 :silence 1: loop play 2: play once
	move.b #0xf,(%a0) ; reset previous command

	RELEASE
	rts	

; *******************************************************************************

	.align 1
	.globl Z80_TempoFM
Z80_TempoFM:
	PAUSE
	move.l  4(%sp), %d0
	
	move.l #FM_TEMPO,%a0
	move.b %d0,(%a0)

	RELEASE
	rts	

; *******************************************************************************
; FUNCTIONS:	DAC handler
; DESCRIPTION:	control of the DAC
; *******************************************************************************	
	.align 1
	.globl Z80_PlayDAC
Z80_PlayDAC:
	PAUSE
	movea.l  4(%sp), %a0 
	move.l  8(%sp), %a1
	move.w %a1, %d1
	move.l %a0, %d0
	move.l #DAC_ADR, %a0
	move.b %d0,(%a0)+
	lsr.l #8,%d0
	move.b %d0,(%a0)+
	lsr.l #8,%d0
	move.b %d0,(%a0)

	move.l #DAC_CMD,%a0
	move.b #1,(%a0) ; 0 :nothing 1: play ff: stop

	move.l #DAC_SIZE, %a0
	move.b %d1, (%a0)+;
	lsr.w #8,%d1
	move.b %d1, (%a0);

	RELEASE
	rts	

; *******************************************************************************

	.align 1
	.globl Z80_StopDAC
Z80_StopDAC:
	PAUSE
	move.l #DAC_CMD,%a0
	move.b #0xFF,(%a0) ; 0 :nothing 1: play ff: stop
	RELEASE
	rts

; *******************************************************************************
; FUNCTIONS:	PSG handler
; DESCRIPTION:	control of the PSG
; *******************************************************************************	
	.align 1
	.globl Z80_PlayPSG
Z80_PlayPSG:
	PAUSE
	movea.l  4(%sp), %a0 
	move.l %a0, %d0
	move.l #PSG_ADR, %a0
	move.b %d0,(%a0)+
	lsr.l #8,%d0
	move.b %d0,(%a0)+
	lsr.l #8,%d0
	move.b %d0,(%a0)

	move.l #PSG_CMD,%a0
	move.b #1,(%a0)+ ; 0 :nothing 1: play ff: stop

	RELEASE
	rts	

; *******************************************************************************
	.align 1
	.globl Z80_StopPSG
Z80_StopPSG:
	PAUSE
	move.l #PSG_CMD,%a0
	move.b #0xFF,(%a0) ; 0 :nothing 1: play ff: stop
	RELEASE
	rts

; *******************************************************************************

	.align 1
	.globl Z80_EnablePSG
Z80_EnablePSG:
	PAUSE
	move.l  4(%sp), %d0
	
	move.l #PSG_CHAN,%a0
	bset.b %d0,(%a0)

	RELEASE
	rts

; *******************************************************************************

	.align 1
	.globl Z80_DisablePSG
Z80_DisablePSG:
	PAUSE
	move.l  4(%sp), %d0
	
	move.l #PSG_CHAN,%a0
	bclr.b %d0,(%a0)

	RELEASE
	rts

; *******************************************************************************
; DATA:	 Z80 Driver
; DESCRIPTION:	Z80 code to load before anything else!
; *******************************************************************************	
	.align 1
Z80Driver:
	INCBIN "md_z80.bin"
Z80DriverEnd:
